////////////////////////////////////////////////////////////////////////////
//
// nrBound.cpp
//
// A class for bounds.
//
// Nate Robins, January 2002.
//
////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////

#include "nrBound.h"

#include "nrInterval.h"
#include "nrRay.h"


////////////////////////////////////////////////////////////////////////////
// Public
////////////////////////////////////////////////////////////////////////////

nrBound::nrBound( const nrVector3& minimums, const nrVector3& maximums )
{
    m_Minimums = minimums;
    m_Maximums = maximums;
}

////////////////////////////////////////////////////////////////////////////

nrBound::~nrBound( void )
{
}

////////////////////////////////////////////////////////////////////////////

bool nrBound::Hit( const nrRay& ray, const nrInterval& interval ) const
{
    const nrVector3& d = ray.d;
    const nrVector3& o = ray.o;
    const nrVector3& p0 = m_Minimums;
    const nrVector3& p1 = m_Maximums;
    
    nrInterval x;
    nrInterval y;
    nrInterval z;
    
    if ( d.x > 0 ) 
    {
        float a = ( p0.x - o.x ) / d.x;
        float b = ( p1.x - o.x ) / d.x;
        x = nrInterval( a, b );
    }
    else
    {
        float a = ( p0.x - o.x ) / d.x;
        float b = ( p1.x - o.x ) / d.x;
        x = nrInterval( b, a );
    }

    if ( d.y > 0 ) 
    {
        float a = ( p0.y - o.y ) / d.y;
        float b = ( p1.y - o.y ) / d.y;
        y = nrInterval( a, b );
    }
    else
    {
        float a = ( p0.y - o.y ) / d.y;
        float b = ( p1.y - o.y ) / d.y;
        y = nrInterval( b, a );
    }
    
    if ( d.z > 0 ) 
    {
        float a = ( p0.z - o.z ) / d.z;
        float b = ( p1.z - o.z ) / d.z;
        z = nrInterval( a, b );
    }
    else
    {
        float a = ( p0.z - o.z ) / d.z;
        float b = ( p1.z - o.z ) / d.z;
        z = nrInterval( b, a );
    }
    
    nrInterval overlap = ( x.Overlap( y ) ).Overlap( z );
    
    /*
    float t0, t1;
    
    if ( x.m_Minimum > y.m_Minimum )
    {
        t0 = x.m_Minimum;
    }
    else
    {
        t0 = y.m_Minimum;
    }
    if ( z.m_Minimum > t0 )
    {
        t0 = z.m_Minimum;
    }
    
    if ( x.m_Maximum < y.m_Maximum )
    {
        t1 = x.m_Maximum;
    }
    else
    {
        t1 = y.m_Maximum;
    }
    if ( z.m_Maximum < t1 )
    {
        t1 = z.m_Maximum;
    }
    */
    
    if ( overlap.Empty() )
    {
        return false;
    }
    else
    {
        return true;
    }
}

////////////////////////////////////////////////////////////////////////////
