////////////////////////////////////////////////////////////////////////////
//
// nrModelLoaderMTL.h
//
// Nate Robins, January 2002.
//
// Wavefront material file format reader.
//
////////////////////////////////////////////////////////////////////////////

#ifndef NRMODELLOADERMTL_H
#define NRMODELLOADERMTL_H


////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////

#include <stdio.h>


////////////////////////////////////////////////////////////////////////////
// Classes
////////////////////////////////////////////////////////////////////////////

class nrVector2;
class nrVector3;
class nrmTriangle;
class nrmGroup;

////////////////////////////////////////////////////////////////////////////

class nrModelLoaderMTL
{
    friend class nrModel;
    
public:
    // Read a Wavefront material file into a model.
    static bool Read( const char* material_file, nrModel* model );
    
private:
    nrModelLoaderMTL();
    ~nrModelLoaderMTL();
    
private:
    // Make a first pass through the MTL model file, taking statistics of 
    // the data (material counts, etc).
    void FirstPass( void );
    
    // Make a second pass through the MTL model file, filling in all the
    // materials.
    void SecondPass( void );
    
private:
    // File reading utilities.
    bool NextLine( void );
    void Rewind( void );
    void Loading( void );
    
//    // Group utilities.
//    nrmGroup* FindGroup( const char* name );
//    void AddTriangleToGroup( nrmGroup* group, int triangle );
//    
//    // Triangle utilities.
//    nrmTriangle* SetTriangle( int triangle, int v0, int n0, int t0, int v1, int n1, int t1, int v2, int n2, int t2 );
//    nrmTriangle* SetNextTriangle( nrmTriangle* p, int triangle, int v2, int n2, int t2 );
    
private:
    nrModel* m_Model;      // Model that we're populating.
    FILE* m_File;          // File from which materials are being read.
    
    char* m_Line;          // Current line in the file.
    int m_LineNum;         // Current line number in the file.
    
    int m_TotalBytes;      // Total number of bytes to be read.
    int m_NumBytes;        // Current number of bytes read.
};

////////////////////////////////////////////////////////////////////////////

#endif    // NRMODELLOADERMTL_H
