////////////////////////////////////////////////////////////////////////////
//
// nrModelLoaderOBJ.h
//
// Nate Robins, November 2001.
//
// Wavefront OBJ model file format reader.
//
////////////////////////////////////////////////////////////////////////////

#ifndef NRMODELLOADEROBJ_H
#define NRMODELLOADEROBJ_H


////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////

#include <stdio.h>


////////////////////////////////////////////////////////////////////////////
// Classes
////////////////////////////////////////////////////////////////////////////

class nrVector2;
class nrVector3;
class nrmTriangle;
class nrmGroup;

////////////////////////////////////////////////////////////////////////////

class nrModelLoaderOBJ
{
    friend class nrModel;
    
public:
    // Read a Wavefront OBJ model file.
    static nrModel* NewFromFile( const char* file_name );
    
private:
    nrModelLoaderOBJ();
    ~nrModelLoaderOBJ();
    
private:
    // Make a first pass through the OBJ model file, taking statistics of 
    // the data (vertex counts, etc).
    void FirstPass( void );
    
    // Make a second pass through the OBJ model file, collecting statistices
    // for the groups.
    void SecondPass( void );
    
    // Make a third pass through the OBJ model file, filling in all the
    // vertex/face data.
    void ThirdPass( void );
    
private:
    // File reading utilities.
    bool NextLine( void );
    void Rewind( void );
    void Loading( void );
    
    // Group utilities.
    nrmGroup* FindGroup( const char* name );
    void AddTriangleToGroup( nrmGroup* group, int triangle );

    // Material utilities.
    int FindMaterial( const char* name );
    
    // Triangle utilities.
    nrmTriangle* SetTriangle( int triangle, int v0, int n0, int t0, int v1, int n1, int t1, int v2, int n2, int t2 );
    nrmTriangle* SetNextTriangle( nrmTriangle* p, int triangle, int v2, int n2, int t2 );
    
private:
    nrModel* m_Model;      // Model that we're populating.
    FILE* m_File;          // File from which model is being populated.

    char* m_Line;          // Current line in the file.
    int m_LineNum;         // Current line number in the file.
    
    int m_TotalBytes;      // Total number of bytes to be read.
    int m_NumBytes;        // Current number of bytes read.
};

////////////////////////////////////////////////////////////////////////////

#endif    // NRMODELLOADEROBJ_H
