////////////////////////////////////////////////////////////////////////////
//
// nrSurfaceBVH.h
//
// A class for a surface bounding volume hierarchy.
//
// Nate Robins, January 2002.
//
////////////////////////////////////////////////////////////////////////////

#ifndef NRSURFACEBVH_H
#define NRSURFACEBVH_H


////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////

#include "nrArray.h"
#include "nrSurface.h"
#include "nrVector3.h"


////////////////////////////////////////////////////////////////////////////
// Classes
////////////////////////////////////////////////////////////////////////////

class nrBound;
class nrHit;
class nrInterval;
class nrRay;

////////////////////////////////////////////////////////////////////////////

class nrSurfaceBVH : public nrSurface
{
public:
    
    nrSurfaceBVH( const nrBound& bound );
    virtual ~nrSurfaceBVH( void );
    
    // Return true if the ray hit the surface, false otherwise.
    virtual bool Hit( const nrRay& ray, nrInterval& interval, nrHit& hit ) const;
    
    // Return a normal to the surface at point (point must be on the surface).
    virtual nrVector3 Normal( const nrVector3& point ) const;

    // Create a hierarchy of bounding volumes from a list of surfaces.
    static nrSurface* CreateTree( nrArray< nrSurface* >& surfaces );
    
private:
    
    // Compare functions which will sort along given axes.
    static int SortInX( const void* _a, const void* _b );
    static int SortInY( const void* _a, const void* _b );
    static int SortInZ( const void* _a, const void* _b );
    
private:
    
    nrSurface* m_Left;
    nrSurface* m_Right;
};

////////////////////////////////////////////////////////////////////////////

#endif  // NRSURFACEBVH_H
